//メッセージ表示関係
var arr_screen_messages = [];
export const delete_all_screen_messages = () =>{
  //lifetimeなしで表示しているメッセージ全削除
  for(let message of arr_screen_messages){
    let container = message.parentNode;
    container.removeChild(message);
  }
  arr_screen_messages = [];
};

var default_container_id = "screen_message";
var default_container_center_id = "screen_message_center";

export const set_screen_message_container = (id) =>{
  default_container_id = id;
};
export const set_screen_message_container_center = (id) =>{
  default_container_center_id = id;
};

const get_default_container = () =>{
  let ret = document.getElementById(default_container_id);
  if(ret){
    return ret;
  }
  //ないので作る
  const el = document.createElement("div");
  el.id = default_container_id;
  return document.body.appendChild(el);
};


const get_default_container_center = () =>{
  let ret = document.getElementById(default_container_center_id);
  if(ret){
    return ret;
  }
  //ないので作る
  const el = document.createElement("div");
  el.id = default_container_center_id;
  return document.body.appendChild(el);

};


export const show_simple_message=(text,lifetime=0)=>{
  let param = {
    content:text,
    container:get_default_container_center(),
    css:{width:'fit-content',height:'fit-content'},
    lifetime:lifetime
  };
  return show_screen_message(param);
}

export const show_simple_button=(text,onclick,lifetime=0,ontimeout=null)=>{
  let param = {
    content:text,
    container:get_default_container_center(),
    css:{width:'fit-content',height:'fit-content',color:"#0000ff"},
    onclick:onclick,
    lifetime:lifetime,
    ontimeout:ontimeout
  };
  return show_screen_message(param);
}


export const show_simple_permanent_button=(text,onclick)=>{
  let param = {
    content:text,
    container:get_default_container_center(),
    css:{width:'fit-content',height:'fit-content',color:"#0000ff"},
    onclick:onclick,
    permanence:true
  };
  return show_screen_message(param);
}



export const show_screen_message=(param)=>{
  /*
  param = {
    content:
    container:,
    layout:{x:, y:, w:, h:}
    css:
    className:
    onclick:  ある場合はbuttonにする
    ontimeout: 時間切れまでにクリックされなかったら呼ばれる
    lifetme:
    permanence:自動削除しない
  }
  */
  //contentはinnerHTML
  //x y は画面中央を基準
  //w h はピクセルサイズ
  //lifetimeは表示時間 省略なら永続
  if(!param){
    return null;
  };
  if(!param.content){
    return null;
  };
  
  let container = null;
  if(param.container){
    container = param.container;
  }else{
    container = get_default_container();
  }
  let eltype = "div";
  if(param.onclick){
    eltype = "button";
  }
  let message = document.createElement(eltype);
  let clicked = false;
  if(param.onclick){
    message.onclick = ()=>{
      clicked = true;
      param.onclick(message);
      if(param.permanence){
        //永続の場合は消さない
      }else{
        container.removeChild(message);
      }
    };
    //ボタンの場合は拾ってほしい
    message.style.pointerEvents="auto";
  }else{
    message.style.pointerEvents="none";//ポインタイベント無視
  }
  message.innerHTML= param.content;
  //css指定なしなら影付きの白テキスト
  message.style.textShadow="1px 1px 2px black";
  message.style.color = "white";
  message.style.fontSize = "30px";
  message.style.position = "absolute";
  
  if(param.scroll){
    let mode = param.scroll.mode;//keyframes
    let dulation = param.scroll.dulation;
    if(!dulation){
      dulation = 2;
    }
    message.style.animation = mode+ " "+dulation+"s infinite"
  }
  
  if(param.css){
    for(let k in param.css){
      message.style[k] = param.css[k];
    }
  }
  if(param.className){
    message.className = param.className;
  };
  container.appendChild(message);
  if(param.lifetime){
    setTimeout(()=>{
      if(param.ontimeout){
        if(!clicked){
          param.ontimeout();
        }
      }
      //lifetime指定がある場合はpermanenceではない
      if(!clicked){
        //クリックしてたら削除済み
        container.removeChild(message);
      }
    },param.lifetime);
  }else{
    //永続属性の場合は呼び出し側で管理する
    if(!param.permanence){
      arr_screen_messages.push(message);
    }
  }
  return message;
}



