import * as THREE from "three";
import { Sky } from "three/examples/jsm/objects/Sky.js";
import { TransformControls } from "three/examples/jsm/controls/TransformControls.js";
import { VRButton } from "three/examples/jsm/webxr/VRButton.js";
import { 
init_light_settings
} from "./roomconf";

export function setupScene(
  tickFunc /* ?: (deltaTime: number) => void */,
  withVR,
  withStats
) {

  const renderer = new THREE.WebGLRenderer({ antialias: true });
  renderer.shadowMap.enabled = true;
  window.the_renderer = renderer;
  window.parent.the_renderer= renderer;
  renderer.outputEncoding = THREE.sRGBEncoding;
  //renderer.physicallyCorrectLights = true;
  //renderer.gammaOutput = true; 
  //renderer.toneMapping = THREE.ACESFilmicToneMapping;

  //renderer.toneMappingExposure = 1;//明るさ？

  renderer.setSize(window.innerWidth, window.innerHeight);
  renderer.setPixelRatio(window.devicePixelRatio);
  document.body.appendChild(renderer.domElement);

  window.addEventListener("resize", () => {
    camera.aspect = window.innerWidth / window.innerHeight;
    camera.updateProjectionMatrix();

    renderer.setSize(window.innerWidth, window.innerHeight);
  });

  const camera = new THREE.PerspectiveCamera(
    60,
    window.innerWidth / window.innerHeight,
    0.1,
    100
  );
  camera.position.set(0.0, 1.5, 2.0);
  const cameraContainer = new THREE.Group();
  cameraContainer.add(camera);

  const scene = new THREE.Scene();
  window.the_scene = scene;
  
  init_light_settings(scene);

  const clock = new THREE.Clock();
  function animate() {

    const dt = clock.getDelta();
    if (tickFunc) {
      tickFunc(dt);
    }
    
    //撮影モード対応
    if(window._useFreeCamera && window.onRender){
      window.onRender(dt);
    }else{
      renderer.render(scene, camera);
    }


  }
  renderer.setAnimationLoop(animate);

  let vrButton = undefined;
  if (withVR) {
    if ("xr" in navigator) {
      navigator.xr
        .isSessionSupported("immersive-vr")
        .then(function (supported) {
          if (supported) {
            renderer.xr.enabled = true;

            document.addEventListener("keydown", function (e) {
              if (e.key === "Escape") {
                if (renderer.xr.isPresenting) {
                  renderer.xr.getSession()?.end();
                }
              }
            });
            //vrButton = VRButton.createButton(renderer);
            //document.body.appendChild(vrButton);
          }
        });
    } else {
      if (window.isSecureContext === false) {
        console.warn("webxr needs https");
      } else {
        console.warn("webxr not available");
      }
    }
  }

  {
    // For Three.js Inspector (https://zz85.github.io/zz85-bookmarklets/threelabs.html)
    window.THREE = THREE;
    window._scene = scene;
  }

  const res = {
    camera,
    scene,
    renderer,
    cameraContainer,
    vrButton,
  };
  window._debugCtx = res; // debug
  return res;
}

