import * as THREE from "three";

export const setupSnow = async (
  scene,renderer,camera
)=>  {

  var getTexture = function () {
    var canvas = document.createElement('canvas');
    canvas.width = 32;
    canvas.height = 32;
    var ctx = canvas.getContext('2d');
    // circlue
    ctx.translate(-81, -84);
    ctx.fillStyle = "white";
    ctx.beginPath();
    ctx.moveTo(91, 96);
    ctx.bezierCurveTo(88, 96, 87, 99, 87, 101);
    ctx.bezierCurveTo(87, 103, 88, 106, 91, 106);
    ctx.bezierCurveTo(94, 106, 95, 103, 95, 101);
    ctx.bezierCurveTo(95, 99, 94, 96, 91, 96);
    ctx.fill();
    var texture = new THREE.Texture(canvas);
    texture.needsUpdate = true;
    return texture;
  };

  var cloud;
  var inited = false;
  var controls = new function () {
    this.size = 3;
    this.transparent = true;
    this.opacity = 0.5;
    this.color = 0x3e6ce1;
    this.rotateSystem = true;
    this.sizeAttenuation = true;
    this.widthRange = 150;
    this.quantity = 10000;
    this.HeightVelocity = 5;
    this.redraw = function () {
      if (scene.getObjectByName("pointcloud")) {
        scene.remove(scene.getObjectByName("pointcloud"));
      }
      createPoints(controls.size,controls.transparent , controls.opacity, controls.sizeAttenuation, controls.color, controls.widthRange, controls.quantity, controls.HeightVelocity);
    };
  };



  var arr_particle = [];
  var vertices  = null;
  var position = null;

  function particle_to_vertices(){
    for (var i = 0; i < arr_particle.length; i++) {
      let particle = arr_particle[i];
      vertices[i*3+0] = particle.x;
      vertices[i*3+1] = particle.y;
      vertices[i*3+2] = particle.z;
    }
  };

  
  function createPoints(size, transparent, opacity, sizeAttenuation, color, widthRange, quantity, HeightVelocity) {
    var geom = new THREE.BufferGeometry();
    var material = new THREE.PointsMaterial({
      size: size,
      transparent: transparent,
      opacity: opacity,
      map: getTexture(),
      blending: THREE.AdditiveBlending,
      sizeAttenuation: sizeAttenuation,
      depthWrite: false,
      color: color
    });

    /*
    var vertices = [
        new THREE.Vector3(0, 0, 0)
    ];
    */
    
    

    var range = widthRange;
    arr_particle = [];
    for (var i = 0; i < quantity; i++) {
      var particle = new THREE.Vector3(
        Math.random() * range - range / 2,
        Math.random() * range * 1.5,
        Math.random() * range - range / 2
      );

      particle.velocityX = (Math.random() - 0.5) / 3;
      particle.velocityY = HeightVelocity / 10;
      arr_particle.push(particle);
    }

    vertices = new Float32Array(quantity * 3);
    particle_to_vertices();
    

    position = new THREE.BufferAttribute(vertices,3);
    geom.setAttribute('position', position);


    cloud = new THREE.Points(geom, material);
    cloud.sortParticles = true;
    cloud.name = 'pointcloud';
    scene.add(cloud);
  }


  function render() {
    if(!window._use_snow){
      return;
    }
    //var vertices = cloud.geometry.vertices;
    arr_particle.forEach(function (v) {
      v.y = v.y - (v.velocityY);
      v.x = v.x - (v.velocityX);
      if (v.y <= 0) v.y = 100;
      if (v.x <= -25 || v.x >= 25) v.velocityX = v.velocityX * -1;
    });
    let position = cloud.geometry.attributes.position;
    for(let i=0;i<arr_particle.length;i++){
      position.setX(i,arr_particle[i].x);
      position.setY(i,arr_particle[i].y);
      position.setZ(i,arr_particle[i].z);
    }
    position.needsUpdate = true;
    
    cloud.geometry.verticesNeedUpdate = true;
    requestAnimationFrame(render);
    //renderer.render(scene, camera);
  }
  
  window.play_snow = ()=>{
    window._use_snow = true;
    controls.redraw();
    render();
  };
  window.stop_snow = ()=>{
    scene.remove(cloud);
    cloud.material.dispose();
    cloud.geometry.dispose();
    
    window._use_snow = false;
  };

};


