import * as THREE from "three";
import { Sky } from "three/examples/jsm/objects/Sky.js";
import { RGBELoader } from 'three/examples/jsm/loaders/RGBELoader.js';
import { 
alertAsync
,confirmAsync 
,choiceAsync
,promptAsync
} from "./dialogUtil";
import { 
geodata
,init_geodata
} from "./roomconf";
import { 
loadModel
,loadModelFromArrayBuffer
,loadPly
,loadPlyFromText
,makePlyGeometryFromText
,faceTo
} from "./modelUtil";

export const prepare_bgm = (id,url) =>{
  //非表示でaudio要素を作る
  const audio = document.createElement("audio");
  audio.src = url;
  audio.id = id;
  document.body.appendChild(audio);
}
export const play_bgm= (id) =>{
  const audio = document.getElementById(id);
  if(audio){
    audio.play();
  }
};
export const stop_bgm= (id) =>{
  const audio = document.getElementById(id);
  if(audio){
    audio.pause();
  }
};


export const setup_default_ambient_light = (scene) =>{
  let anbient_light = new THREE.AmbientLight(0xffffff, 1);
  window.anbient_light = anbient_light;
}

window.set_a_light_intensity = (v) =>{
  window.anbient_light.intensity = v;
}
window.set_a_light_color = (v) =>{
  window.anbient_light.color = new THREE.Color(v);
}

export const setup_default_directional_light = (scene) =>{
  //直線光
  var light = new THREE.DirectionalLight(0xFFFFFF,1);
  window.directional_light = light;
  light.position.set(0, 30, 20);
  light.shadow.mapSize.width = 5120 // default
  light.shadow.mapSize.height = 5120 // default
  light.shadow.camera.near = 0.1 // default
  light.shadow.camera.far = 500 // default
  light.shadow.camera.top = -100 // default
  light.shadow.camera.right = 100 // default
  light.shadow.camera.left = -100 // default
  light.shadow.camera.bottom = 100 // default
  light.castShadow = true; 
  var light_target = new THREE.Object3D();
  scene.add(light_target);
  window.light_target = light_target;
  light.target = light_target;
  scene.add(light);
}

window.set_d_light_intensity = (v) =>{
  window.directional_light.intensity = v;
}
window.set_d_light_color = (v) =>{
  window.directional_light.color = new THREE.Color(v);
}

window.create_spotlight = (origin,target,intensity=1)=>{
  let spotlight = new THREE.SpotLight(0xFFFFFF, 4, 30, Math.PI / 4, 10, 0.5);
  //todo なぜか影を有効にするとシェーダエラーが発生する
  //three-vrmのfragmentシェーダでvSpotShadowCoord これが未定義のためなんだが
  //仕方が無いのでシェーダを書き換えたmoduleをローカルにおいている
  //threeのr153だと動かない。r154指定。
  spotlight.castShadow = true;
  spotlight.shadow.mapSize.width = 1024;
  spotlight.shadow.mapSize.height = 1024;
  spotlight.shadow.camera.near = 0.5;
  spotlight.shadow.camera.far = 4000;
  spotlight.intensity = intensity;
  window.the_scene.add(spotlight);
  spotlight.position.set(origin.x,origin.y,origin.z);
  if(target.position){
    spotlight.target = target;
  }else{
    let dummy_target = new THREE.Object3D();
    window.the_scene.add(dummy_target);
    dummy_target.position.set(target.x,target.y,target.z);
    spotlight.target = dummy_target;
  }
  return spotlight;
}

export const setup_default_sky_and_sun = (scene) =>{
  const sky = new Sky();
  sky.scale.setScalar(450000);
  scene.add(sky);
  const sky_uniforms = sky.material.uniforms;
  sky_uniforms['turbidity'].value = 10;
  sky_uniforms['rayleigh'].value = 2;
  sky_uniforms['mieCoefficient'].value = 0.005;
  sky_uniforms['mieDirectionalG'].value = 0.8;
  
  window.sky = sky;
  window.sky_uniforms = sky_uniforms;
  window.set_sky_rayleigh = (val) =>{
    sky_uniforms['rayleigh'].value = val;
  };
  window.set_sun_height = (ht) =>{
    sky_uniforms.sunPosition.value.y=ht
  };

  const sun = new THREE.Vector3();
  sun.setFromSphericalCoords(
    1,
    THREE.MathUtils.degToRad(60),
    THREE.MathUtils.degToRad(180)
  );
  sky.material.uniforms["sunPosition"].value.copy(sun);
  
  window.hide_sky=()=>{
    sky.visible=false;
  };
  
  window.show_sky=()=>{
    sky.visible=true;
  };

  let dic_hdri_texture = {};
  window.hide_hdri=() =>{
    sky.visible=true;
    scene.background = null;//微妙か？      
  };

  window.show_hdri=(url) =>{
    if(url in dic_hdri_texture){
      sky.visible=false;
      scene.background = dic_hdri_texture[url];
      return;
    }else{
      new RGBELoader().load(url, function (texture) {
        texture.mapping = THREE.EquirectangularReflectionMapping;
        dic_hdri_texture[url] = texture;
        console.log("hdr loaded");
        sky.visible=false;
        scene.background = texture; // 解像度の高いテクスチャを使用
        //scene.environment = texture;
      });

    }
  }
}





export async function createWorldObjects({
  scene,
  collisionObjects,
  _interactableObjects,
  teleportTargetObjects,
}) {
  
  //動的に移動地形追加 
  //自前でメッシュを作ってから呼ぶ
  //todo 機能するか検証する
  window.append_teleportTargetObjects = (o) =>{
    teleportTargetObjects.push(o);
  };
  window.append_box_land = (geos) =>{
    //geod 配置情報の配列
    let gd_idx = geodata.length;
    for(let gd of geos){
      geodata.push(gd);
      const box = new THREE.Mesh(
        new THREE.BoxGeometry(gd.sx, gd.sy, gd.sz),
        new THREE.MeshLambertMaterial({
          color: gd.color,
          side: THREE.DoubleSide,
        })
      );
      box.receiveShadow = true;
      box.castShadow = true;
      
      gd_idx++;
      if(gd.name){
        box.name = gd.name;
      }else{
        box.name = "box_"+gd_idx;
      }
      box.position.set(gd.px, gd.py, gd.pz);
      the_scene.add(box);
      teleportTargetObjects.push(box);
    }
    //後から地形追加する場合は再計算必要
    window.fix_collision();
  };
  
  
  //geodataから地形
  init_geodata();//
  let gd_idx = 0;
  for(const gd of geodata){
    if(gd.type=="box"){

      const box = new THREE.Mesh(
        new THREE.BoxGeometry(gd.sx, gd.sy, gd.sz),
        new THREE.MeshLambertMaterial({
          color: gd.color,
          side: THREE.DoubleSide,
        })
      );
      box.receiveShadow = true;
      //box.castShadow = true;

      gd_idx++;
      if(gd.name){
        box.name = gd.name;
      }else{
        box.name = "box_"+gd_idx;
      }
      box.position.set(gd.px, gd.py, gd.pz);

      scene.add(box);

      //ロード時処理
      if(gd.onLoad){
        gd.onLoad(box);
      }


      //クリックイベントの設定
      if(gd.onSelectDown){
        box.onSelectDown = gd.onSelectDown;
        window.arr_interactive.push(box);
      }
      if(gd.collision=="wall"){
        //障害物
        collisionObjects.push(box);
      }else if(gd.collision=="none"){
        //見た目だけで判定なし
      }else{
        //移動可能地形
        teleportTargetObjects.push(box);
      }

    }else if(gd.type=="plane"){
      const obj  = new THREE.Mesh(
        new THREE.PlaneGeometry(gd.sx, gd.sz, 1, 1),
        new THREE.MeshLambertMaterial({
          color: gd.color,
        })
      );
      obj.receiveShadow = true;
      //obj.castShadow = true;


      //xz平面のみ
      obj.rotation.x = Math.PI / -2;
      obj.receiveShadow = true;//影落とす
      gd_idx++;
      if(gd.name){
        obj.name = gd.name;
      }else{
        obj.name = "plane_"+gd_idx;
      }
      obj.position.set(gd.px, gd.py, gd.pz);
      scene.add(obj);

      //ロード時処理
      if(gd.onLoad){
        gd.onLoad(obj);
      }
      //クリックイベントの設定
      if(gd.onSelectDown){
        box.onSelectDown = gd.onSelectDown;
        window.arr_interactive.push(obj);
      }
      if(gd.collision=="wall"){
        //障害物
        collisionObjects.push(obj);
      }else if(gd.collision=="none"){
        //見た目だけで判定なし
      }else{
        //移動可能地形
        teleportTargetObjects.push(obj);
      }

    }else if(gd.type=="glb"){
      let _glb = await loadModel(gd.resource);
      const glb = _glb.scene;
      const obj = new THREE.Object3D();
      obj.add(glb);
      obj.actions = {};

      gd_idx++;
      if(gd.name){
        obj.name = gd.name;
      }else{
        obj.name = "glb_"+gd_idx;
      }


      //埋め込みアニメーション対応
      const animations = _glb.animations;
      if( animations && animations.length) {
        //Animation Mixerインスタンスを生成
        let mixer = new THREE.AnimationMixer(obj);
        //全てのAnimation Clipに対して
        for (let i = 0; i < animations.length; i++) {
            let animation = animations[i];
            //Animation Actionを生成
            let action = mixer.clipAction(animation) ;
            //アニメーションの最後のフレームでアニメーションが終了
            action.clampWhenFinished = true;
            //アニメーションを再生
            if(gd.animation =="auto"){
              action.play();
            }
            obj.actions[animation.name] = action;

        }
        //静的なので削除は考慮しない
        let anifn = function(dt){
          mixer.update(dt);
        }
        window.animation_ticks.push(anifn);
      }

      obj.scale.set(gd.sx, gd.sy, gd.sz);
      obj.position.set(gd.px, gd.py, gd.pz);
      obj.rotation.set(gd.rx, gd.ry, gd.rz);
      obj.traverse(
        (object) => { //モデルの構成要素
          if(object.isMesh) { //その構成要素がメッシュだったら
            object.castShadow = true;//影表示
            //object.receiveShadow = true;//影表示
            object.material.metalness = 0;//これしないと暗くなる
          }
        }
      );
      scene.add(obj);

      //ロード時処理
      if(gd.onLoad){
        gd.onLoad(obj);
      }


      //クリックイベントの設定
      if(gd.onSelectDown){
        obj.onSelectDown = gd.onSelectDown;
        window.arr_interactive.push(obj);
      }
      if(gd.collision=="wall"){
        //障害物
        collisionObjects.push(obj);
      }else if(gd.collision=="none"){
        //見た目だけで判定なし
      }else{
        //移動可能地形
        teleportTargetObjects.push(obj);
      }
    }else if(gd.type=="ply"){
      let obj = await loadPly(gd.resource);
      obj.scale.set(gd.sx, gd.sy, gd.sz);
      obj.position.set(gd.px, gd.py, gd.pz);
      obj.rotation.set(gd.rx, gd.ry, gd.rz);


      obj.traverse(
        (object) => { //モデルの構成要素
          if(object.isMesh) { //その構成要素がメッシュだったら
            object.castShadow = true;//影表示
            //object.receiveShadow = true;//影表示
            object.material.metalness = 0;//これしないと暗くなる
          }
        }
      );

      gd_idx++;
      if(gd.name){
        obj.name = gd.name;
      }else{
        obj.name = "glb_"+gd_idx;
      }
      
      scene.add(obj);


      //ロード時処理
      if(gd.onLoad){
        gd.onLoad(obj);
      }

      //クリックイベントの設定
      if(gd.onSelectDown){
        obj.onSelectDown = gd.onSelectDown;
        window.arr_interactive.push(obj);
      }
      if(gd.collision=="wall"){
        //障害物
        collisionObjects.push(obj);
      }else if(gd.collision=="none"){
        //見た目だけで判定なし
      }else{
        //移動可能地形
        teleportTargetObjects.push(obj);
      }
    }
  }

}
